<?php


namespace kmcf7_message_filter;

class ExtensionsModule extends Module {
	private $prevent_default_validation;

	public function __construct() {
		parent::__construct();
		$this->prevent_default_validation = get_option( 'kmcfmf_hide_error_message' ) == 'on' ? true : false;

	}

	/**
	 * Add field to toggle this module
	 *
	 * @param $settings \KMSetting
	 *
	 * @since v1.3.6
	 */
	public function addSettingsField( $settings ) {
		$settings->add_field(
			array(
				'type'  => 'checkbox',
				'id'    => 'kmcfmf_hide_error_message',
				'label' => __( 'Hide error messages: ', KMCF7MS_TEXT_DOMAIN ),
				'tip'   => __( "Show a success message instead of an error message if a spam is found", KMCF7MS_TEXT_DOMAIN )
			)
		);

		return $settings;
	}

	/**
	 * Prevent default validation if a spam is found
	 *
	 * @param $validate boolean
	 *
	 * @since v1.3.6
	 */
	public function preventDefaultValidation( $validate ) {
		if ( $this->prevent_default_validation ) {
			$this->removeActions();

			return false;
		}

		return $validate;
	}

	/**
	 * Removes contact form 7 submit actions
	 * @since v1.3.6
	 */
	private function removeActions() {
		remove_all_actions( 'wpcf7_mail_sent' );
		remove_all_actions( 'wpcf7_before_send_mail' );
	}

	/**
	 * Skips sending of contact form mail
	 * @since v1.3.6
	 */
	function skipMail( $skip_mail, $contact_form ) {
		global $kmcf7_spam_status;

		if ( $this->prevent_default_validation && $kmcf7_spam_status ) {

			return true;
		}

		return $skip_mail;
	}

	/**
	 * Registers this extension
	 *
	 * @param $extensions array
	 *
	 * @since v1.3.6
	 */
	public function registerExtension( $extensions ) {
		array_push( $extensions, 'hide_error_messages' );

		return $extensions;
	}

	/**
	 * @since v1.3.6
	 */
	protected function addFilters() {
		parent::addFilters();
		add_filter( 'kmcf7_message_settings', array( $this, 'addSettingsField' ), 10, 1 );
		add_filter( 'kmcf7_invalidate_text_field', array( $this, 'preventDefaultValidation' ), 10, 1 );
		add_filter( 'kmcf7_invalidate_email_field', array( $this, 'preventDefaultValidation' ), 10, 1 );
		add_filter( 'wpcf7_skip_mail', array( $this, 'skipMail' ), 999, 2 );
		add_filter( 'kmcf7_extensions', array( $this, 'registerExtension' ) );

		// add actions here
	}
}